<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    xmlns:doc="http://www.saxonica.com/ns/doc" xmlns:f="MyFunctions"
    xmlns:svg="http://www.w3.org/2000/svg" xmlns:jwl="http://jwlresearch.net/"
    xmlns:saxon="http://saxon.sf.net/" exclude-result-prefixes="xs math f doc" version="3.0">
    <xsl:param name="CSS.file">../../../mystyle.css</xsl:param>

    <xsl:import href="grid.xsl"/>
    <xsl:include href="svg-graphics.xsl"/>
    <xsl:include href="../lib/css2xslt.xsl"/>
    <xsl:variable name="width" as="xs:integer" select="1000"/>
    <xsl:variable name="height" as="xs:integer" select="500"/>

    <xsl:function name="svg:makeKey" as="element()" xmlns="http://www.w3.org/2000/svg"
        expand-text="yes" doc:doc="Create a graph key element">
        <xsl:param name="cases" as="element(case)*"/>
        <svg x="300" y="130">
            <xsl:for-each select="$cases">
                <circle cx="5" cy="{20*position() - 8}" r="5" class="{@name}" stroke="none"/>
                <text x="20" y="{20*position()}" class="key label">{(@label,@name)[1]}</text>
            </xsl:for-each>
        </svg>
    </xsl:function>

    <xsl:function name="svg:scale" as="element()*"
        doc:doc="Apply a scaling transformation to some graphics,
        by altering co-ordinates, rather than wrapping in an SVG affine transform.
        This permits constant line width in the result.">
        <xsl:param name="content" as="element()*" doc:doc="The graphic elements to transform"/>
        <xsl:param name="transform" as="map(*)" doc:doc="The transform to use"/>
        <xsl:apply-templates select="$content" mode="svg:scale">
            <xsl:with-param name="x.scale" select="$transform('scale.x')" tunnel="yes"/>
            <xsl:with-param name="y.scale" select="-$transform('scale.y')" tunnel="yes"/>
            <xsl:with-param name="y.offset" select="$height" tunnel="yes"/>
        </xsl:apply-templates>
    </xsl:function>

    <xsl:template name="svg:makePart" as="element()*" expand-text="yes"
        doc:doc="Generate an SVG portion" xmlns="http://www.w3.org/2000/svg">
        <xsl:param name="g" as="map(*)"/>
        <xsl:param name="content" select="()"/>
        <xsl:param name="content.unscaled" select="()"/>
        <xsl:param name="width" as="xs:integer" select="$width"/>
        <xsl:param name="height" as="xs:integer" select="$height"/>
        <xsl:variable name="grid" select="($g('grid'),svg:makeGrid#3)[1]" as="function(*)"
            doc:doc="The default is a full grid"/>
        <g transform="translate({($g('x'),0)[1]},{($g('y'),0)[1]})">
            <xsl:if test="exists($content)">
                <xsl:sequence select="$grid($g, $width, $height)"/>
            </xsl:if>
            <xsl:sequence select="svg:scale($content, $g)"/>
            <xsl:sequence select="$content.unscaled"/>
        </g>
    </xsl:template>

    <xsl:template name="svg:makeSVG" expand-text="yes" doc:doc="Generate an SVG report picture">
        <xsl:param name="svg.root" as="xs:string"/>
        <xsl:param name="name" as="xs:string"/>
        <xsl:param name="mode" as="xs:string"/>
        <xsl:param name="CSS.file" select="$CSS.file" tunnel="yes"/>
        <xsl:param name="g" as="map(*)"/>
        <xsl:param name="content" select="()"/>
        <xsl:param name="content.unscaled"/>
        <xsl:param name="date.stamp" as="xs:boolean" select="true()"/>
        <xsl:param name="width" as="xs:integer" select="$width"/>
        <xsl:param name="height" as="xs:integer" select="$height"/>
        <xsl:param name="margin" as="xs:integer" select="40"/>
        <xsl:variable name="margin.top"
            select="
                if ($date.stamp) then
                    $margin
                else
                    $margin div 2"/>
        <xsl:variable name="margin.left" select="$margin"/>
        <xsl:variable name="grid" select="($g('grid'),svg:makeGrid#3)[1]" as="function(*)"
            doc:doc="The default is a full grid"/>

        <xsl:variable name="svg" as="element()">
            <svg id="{$mode}/{$name}" xmlns="http://www.w3.org/2000/svg" width="100%" height="100%"
                viewBox="0 0 {xs:integer(1.1 * $width)} {xs:integer(1.15 *$height)}">
                <desc/>
                <xsl:if test="$date.stamp">
                    <text x="20" y="20" font-size="10">
                        <xsl:value-of select="current-dateTime()"/>
                    </text>
                </xsl:if>
                <g alignment-baseline="baseline" transform="translate({$margin.left},{$margin.top})">
                    <xsl:call-template name="svg:makePart">
                        <xsl:with-param name="g" select="$g"/>
                        <xsl:with-param name="content" select="$content"/>
                        <xsl:with-param name="height" select="$height"/>
                        <xsl:with-param name="width" select="$width"/>
                        <xsl:with-param name="content.unscaled" as="element()*">
                            <xsl:variable name="height"
                                select="xs:integer(($g('box')('height'),$height)[1])"/>
                            <xsl:variable name="width"
                                select="xs:integer(($g('box')('width'),$width)[1])"/>
                            <g
                                transform="translate({($g('label.x'),$width div 2)[1]},{($g('label.y'),$height div 2)[1]})">
                                <xsl:for-each select="$g('label')">
                                    <text class="label" x="0" y="{20 * position()}">
                                        <xsl:value-of select="."/>
                                    </text>
                                </xsl:for-each>
                            </g>
                            <xsl:sequence select="$g('ABCDEF')"/>
                        </xsl:with-param>
                    </xsl:call-template>
                    <xsl:sequence select="$content.unscaled"/>
                </g>
            </svg>
        </xsl:variable>

        <xsl:variable name="stylesheet.attributes" as="attribute()*" expand-text="no">
            <xsl:attribute name="xpath-default-namespace">http://www.w3.org/2000/svg</xsl:attribute>
        </xsl:variable>
        <xsl:variable name="compiled.css"
            select="jwl:compileCSS(resolve-uri($CSS.file, resolve-uri(concat($svg.root, '/modes/name/'), $base-uri)), $stylesheet.attributes)"/>
        <xsl:variable name="flat" select="saxon:transform($compiled.css, $svg,())/*"/>
        <xsl:sequence select="$flat"/>
    </xsl:template>
    
    <xsl:template name="svg:makeSVGFile" expand-text="yes" doc:doc="Generate an SVG report picture">
        <xsl:param name="svg.root" as="xs:string"/>
        <xsl:param name="name" as="xs:string"/>
        <xsl:param name="mode" as="xs:string"/>
         <xsl:param name="svg" as="element()*"/>
        <xsl:result-document href="{$svg.root}/modes/{f:modeFile($mode)}/{$name}.svg">
           <!-- <xsl:processing-instruction name="xml-stylesheet">href="{$CSS.file}" type=""text/css"</xsl:processing-instruction>-->
            <xsl:sequence select="$svg"/>
        </xsl:result-document>
    </xsl:template>
    
    <xsl:template name="svg:makeFile" expand-text="yes" doc:doc="Generate an SVG report picture">
        <xsl:param name="svg.root" as="xs:string"/>
        <xsl:param name="name" as="xs:string"/>
        <xsl:param name="mode" as="xs:string"/>
        <xsl:param name="CSS.file" select="$CSS.file" tunnel="yes"/>
        <xsl:param name="g" as="map(*)"/>
        <xsl:param name="content" select="()"/>
        <xsl:param name="content.unscaled"/>
        <xsl:param name="date.stamp" as="xs:boolean" select="true()"/>
        <xsl:param name="width" as="xs:integer" select="$width"/>
        <xsl:param name="height" as="xs:integer" select="$height"/>
        <xsl:param name="margin" as="xs:integer" select="40"/>
        <xsl:variable name="margin.top"
            select="
            if ($date.stamp) then
            $margin
            else
            $margin div 2"/>
        <xsl:variable name="margin.left" select="$margin"/>
        <xsl:variable name="grid" select="($g('grid'),svg:makeGrid#3)[1]" as="function(*)"
            doc:doc="The default is a full grid"/>
        
        <xsl:variable name="svg" as="element()">
            <svg xmlns="http://www.w3.org/2000/svg" width="100%" height="100%"
                viewBox="0 0 {xs:integer(1.1 * $width)} {xs:integer(1.15 *$height)}">
                <desc/>
                <xsl:if test="$date.stamp">
                    <text x="20" y="20" font-size="10">
                        <xsl:value-of select="current-dateTime()"/>
                    </text>
                </xsl:if>
                <g alignment-baseline="baseline" transform="translate({$margin.left},{$margin.top})">
                    <xsl:call-template name="svg:makePart">
                        <xsl:with-param name="g" select="$g"/>
                        <xsl:with-param name="content" select="$content"/>
                        <xsl:with-param name="height" select="$height"/>
                        <xsl:with-param name="width" select="$width"/>
                        <xsl:with-param name="content.unscaled" as="element()*">
                            <xsl:variable name="height"
                                select="xs:integer(($g('box')('height'),$height)[1])"/>
                            <xsl:variable name="width"
                                select="xs:integer(($g('box')('width'),$width)[1])"/>
                            <g
                                transform="translate({($g('label.x'),$width div 2)[1]},{($g('label.y'),$height div 2)[1]})">
                                <xsl:for-each select="$g('label')">
                                    <text class="label" x="0" y="{20 * position()}">
                                        <xsl:value-of select="."/>
                                    </text>
                                </xsl:for-each>
                            </g>
                            <xsl:sequence select="$g('ABCDEF')"/>
                        </xsl:with-param>
                    </xsl:call-template>
                    <xsl:sequence select="$content.unscaled"/>
                </g>
            </svg>
        </xsl:variable>
        
        <xsl:variable name="stylesheet.attributes" as="attribute()*" expand-text="no">
            <xsl:attribute name="xpath-default-namespace">http://www.w3.org/2000/svg</xsl:attribute>
        </xsl:variable>
        <xsl:variable name="compiled.css"
            select="jwl:compileCSS(resolve-uri($CSS.file, resolve-uri(concat($svg.root, '/modes/name/'), $base-uri)), $stylesheet.attributes)"/>
        <xsl:variable name="flat" select="saxon:transform($compiled.css, $svg,())/*"/>
        
        <output href="{$svg.root}/modes/{f:modeFile($mode)}/{$name}.flat.svg"/>
        
        <xsl:result-document href="{$svg.root}/modes/{f:modeFile($mode)}/{$name}.flat.svg">
            <xsl:sequence select="$flat"/>
        </xsl:result-document>
        
        <xsl:result-document href="{$svg.root}/modes/{f:modeFile($mode)}/{$name}.svg">
            <xsl:processing-instruction name="xml-stylesheet">href="{$CSS.file}" type=""text/css"</xsl:processing-instruction>
            <xsl:sequence select="$svg"/>
        </xsl:result-document>
    </xsl:template>

    <xsl:function name="f:modeFile" as="xs:string"
        doc:doc="produce a safe filename from a mode name">
        <xsl:param name="mode" as="xs:string"/>
        <xsl:value-of select="replace($mode, '[:_#]+', '-')"/>
    </xsl:function>

    <xsl:function name="f:graphics.vals" as="map(*)">
        <xsl:param name="graphics"/>
        <xsl:sequence
            select="
                map:merge((f:graphics.vals($graphics, $width, $height),
                $labelling.defaults))"
        />
    </xsl:function>
    <xsl:function name="f:graphics.vals" as="map(*)">
        <xsl:param name="graphics"/>
        <xsl:param name="width" as="xs:double"/>
        <xsl:param name="height" as="xs:double"/>
        <xsl:sequence
            select="
            map:merge((f:graphics.vals($graphics, map {'x' : 0, 'y' : 0, 'width' : $width,'height' : $height}),
            $labelling.defaults))"
        />
    </xsl:function>
    <xsl:function name="f:scale.vals" as="map(*)"
        doc:doc="Generate suitable scalings for a set of graph elements.">
        <xsl:param name="graphics" as="element()+"/>
        <xsl:param name="box" as="map(*)"/>
        <xsl:variable name="x" select="$box('x')" as="xs:double"/>
        <xsl:variable name="y" select="$box('y')" as="xs:double"/>
        <xsl:variable name="width" select="$box('width')" as="xs:double"/>
        <xsl:variable name="height" select="$box('height')" as="xs:double"/>

        <xsl:variable name="max.y" select="xs:integer(svg:max.y($graphics))"/>
        <xsl:variable name="max.x" select="xs:integer(svg:max.x($graphics))"/>

        <xsl:variable name="scale.x" select="$width div $max.x"/>
        <xsl:variable name="scale.y" select="$height div $max.y"/>

        <xsl:sequence
            select="map{
            'box' : $box,
            'x' : $x, 'y' : $y,
            'scale.x' : $scale.x, 'scale.y' : $scale.y,
            'max.x' : $max.x , 'max.y' : $max.y   
            }"
        />
    </xsl:function>
    <xsl:function name="f:graphics.vals" as="map(*)"
        doc:doc="Generate suitable scalings and grid/axis values for a set of graph elements.">
        <xsl:param name="graphics" as="element()+"/>
        <xsl:param name="box" as="map(*)"/>
        <xsl:variable name="x" select="$box('x')" as="xs:double"/>
        <xsl:variable name="y" select="$box('y')" as="xs:double"/>
        <xsl:variable name="width" select="$box('width')" as="xs:double"/>
        <xsl:variable name="height" select="$box('height')" as="xs:double"/>

        <xsl:variable name="max.y" select="xs:integer(svg:max.y($graphics))"/>
        <!--<xsl:message select="'max.y:', $max.y"/>-->
        <xsl:variable name="unit.y" select="max((svg:axis-minor-tick(0, $max.y), 1))"/>
        <xsl:variable name="n.units.y" select="$max.y idiv $unit.y + 1"/>
        <xsl:variable name="max.y" select="$unit.y * $n.units.y"/>
        <!--<xsl:message select="'max.y*:', $max.y"/>-->

        <xsl:variable name="max.x" select="xs:integer(svg:max.x($graphics))"/>
        <xsl:variable name="unit.x" select="max((svg:axis-minor-tick(0, $max.x), 1))"/>
        <xsl:variable name="n.units.x" select="$max.x idiv $unit.x + 1"/>
        <xsl:variable name="max.x" select="$unit.x * $n.units.x"/>

        <xsl:variable name="scale.x" select="$width div $max.x"/>
        <xsl:variable name="scale.y" select="$height div $max.y"/>

        <xsl:sequence
            select="map{
            'box' : $box,
                'x' : $x, 'y' : $y,
                'scale.x' : $scale.x, 'scale.y' : $scale.y,
                'max.x' : $max.x, 'unit.x' : $unit.x,  'n.units.x' : $n.units.x,
                'max.y' : $max.y, 'unit.y' : $unit.y,   'n.units.y' : $n.units.y      
                }"
        />
    </xsl:function>

    <xsl:variable name="labelling.defaults"
        select="map{
        'axis.x.format': format-number(?,'0'),
        'axis.y.format': format-number(?,'0'),
        'axis.x.label' : '',
        'axis.y.label' : '',
        'label' : ''
        }"
    />
</xsl:stylesheet>
